/***************************************************************************************************
	© 2013 McGladrey LLP  All rights reserved.

McGladrey LLP (McGladrey) grants to Client a limited, non-exclusive, non-transferable right to
install and use the Software/Source Code solely for internal business purposes.  Client may not
distribute, give, rent, loan, or resell a copy of the Software/Source Code to any third party.
McGladrey is the owner of the copyright for the Software/Source Code and McGladrey reserves all
rights related to such ownership.  Any Warranty granted by McGladrey is void if Software/Source Code
is modified by anyone other than McGladrey.

$URL: https://svn.mcgladreycloud.com/repos/NonClient/CSA_iPart/trunk/CSA/SQL/CSA_install.sql $
$Rev: 7357 $
$Date: 2013-06-21 14:42:15 -0500 (Fri, 21 Jun 2013) $
***************************************************************************************************/
-- Creates SQL objects used by Customer Service Alerts

set nocount on

print 'Creating table tcsa_AlertDefinitions'
go
-- 1.  Test if tcsa_status_rules exists (only for migrating)
--		*  Create new table, migrate old data, drop old table
-- 2.  Test if tcsa_AlertDefinitions exists (to preserve data)
--		*  Rename old, create new, copy data over, drop old

if object_id('tcsa_status_rules') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_status_rules') and indid < 2) > 0
	begin
		exec sp_rename tcsa_status_rules, tcsa_status_rules_old
		alter table tcsa_status_rules_old
			drop constraint pkcsa_status_rules
	end
	else
	begin
		drop table tcsa_status_rules
	end
end

if object_id('tcsa_AlertDefinitions') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_AlertDefinitions') and indid < 2) > 0
	begin
		exec sp_rename tcsa_AlertDefinitions, tcsa_status_rules_old
		alter table tcsa_status_rules_old
			drop constraint pkcsa_AlertDefinitions
	end
	else
	begin
		drop table tcsa_AlertDefinitions
	end
end

create table tcsa_AlertDefinitions
(
	SEQN int identity
		constraint pkcsa_AlertDefinitions primary key clustered,
	CRITERIA varchar(3072) not NULL default '',
	ENABLED bit not NULL default 1,
	DESCRIPTION varchar(255) not NULL default '',
	ALERT_TYPE varchar(10) not NULL default 'SQL',
	ALERT_ID varchar(20) not NULL default '',
	RuleGroup varchar(100) not NULL default ''
)
go
grant select, insert, update, delete on tcsa_AlertDefinitions to iMIS
go

-- Need a copy of this block for both old table name and new table name
if object_id('tcsa_status_rules_old') is not NULL
begin
	declare @oldrows int,
		@newrows int
	set identity_insert tcsa_AlertDefinitions on

	if exists (select 1 from syscolumns where object_name(id) = 'tcsa_status_rules_old' and name = 'RuleGroup')
	begin
		insert tcsa_AlertDefinitions (SEQN, CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
			select SEQN, CRITERIA, ENABLED, '', 'SQL', 'CSA1 alert ' + convert(varchar, SEQN), RuleGroup
			from tcsa_status_rules_old
	end
	else
	begin
		insert tcsa_AlertDefinitions (SEQN, CRITERIA, ENABLED, DESCRIPTION, ALERT_TYPE, ALERT_ID, RuleGroup)
			select SEQN, CRITERIA, ENABLED, '', 'SQL', 'CSA1 alert ' + convert(varchar, SEQN), ''
			from tcsa_status_rules_old
	end

	set @newrows = @@rowcount

	select @oldrows = count(*)
		from tcsa_status_rules_old

	if @oldrows = @newrows
	begin
		drop table tcsa_status_rules_old
	end
	else
	begin
		print '!!! Error copying data into new table'
	end

	set identity_insert tcsa_AlertDefinitions off
end
go
exec sp_recompile 'tcsa_AlertDefinitions'
go

print 'Creating table tcsa_AlertOutput (Xtender version)'
go
if object_id('tcsa_Status_Output') is not NULL
begin
	-- Always ok to delete
	drop table tcsa_Status_Output
end
go
if object_id('tcsa_AlertOutput') is not NULL
begin
	-- Always ok to delete
	drop table tcsa_AlertOutput
end
go
create table tcsa_AlertOutput
(
	SEQN int identity
		constraint pkcsa_AlertOutput primary key clustered,
	OWNER_SPID smallint not NULL default @@spid,
	IMIS_USER varchar(50) not NULL default user_name(),
	EVENT_TYPE varchar(15) not NULL default 'ID',
	KEY_VALUE varchar(31) not NULL default '',
	RANK smallint not NULL default 0,						-- main window
	ICON smallint not NULL default 4,						-- detail
	KEYWORD varchar(40) not NULL default '',				-- detail
	DESCRIPTION varchar(4000) not NULL default '',			-- detail
	TOOLTIP_TEXT varchar(1000) not NULL default '',			-- detail
	TITLE_TEXT varchar(255) not NULL default '',			-- detail
	CLIPBOARD_TEXT text NOT NULL default '',				-- detail
	CLIPBOARD_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TEXT varchar(1000) not NULL default '',			-- detail
	ACTION_DESCRIPTION varchar(100) not NULL default '',	-- detail
	ACTION_TYPE varchar(30) not NULL default 'Launch',		-- detail
	LOG_TEXT varchar(1000) not NULL default '',				-- main window
	PLAY_SOUND bit not NULL default 0,						-- main window
	AUTO_POPUP bit not NULL default 0,						-- detail
	AUTO_CLIPBOARD bit not NULL default 0,					-- detail
	AUTO_ACTION bit not NULL default 0						-- detail
)
go
create index icsa_AlertOutputUSERRANK
	on tcsa_AlertOutput(OWNER_SPID, RANK)
go
grant select, insert, update, delete on tcsa_AlertOutput to iMIS
go


print 'Creating procedure pcsa_WriteSystemParams (Xtender version)'
go
if object_id('pcsa_WriteSystemParams') is not NULL
begin
	drop procedure pcsa_WriteSystemParams
end
go
create procedure pcsa_WriteSystemParams
	@sParameterName varchar(50),
	@sShortValue varchar(255)
as
begin
	set nocount on

	if @sParameterName is not NULL
	begin
		if @sShortValue is NULL
		begin
			delete System_Params
				where ParameterName = @sParameterName
		end
		else
		begin
			insert System_Params (ParameterName, ShortValue)
				select @sParameterName, @sShortValue
				where not exists (select * from System_Params where ParameterName = @sParameterName)
			
			if @@rowcount = 0
			begin
				update sp
					set ShortValue = @sShortValue
					from System_Params sp
					where sp.ParameterName = @sParameterName
			end
		end
	end
end
go
grant execute on pcsa_WriteSystemParams to IMIS
go



print 'Dropping obsolete procedure pcsa_calculate_status'
go
if object_id('pcsa_calculate_status') is not NULL
begin
	drop procedure pcsa_calculate_status
end
go

print 'Adding security keyword for CSA Editor window (for Xtender version)'
go
if not exists (select * from Gen_Tables where TABLE_NAME = 'ACCESS_KEYWORDS' 
		and upper(CODE) = 'CSAEDIT')
begin
	insert Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, DESCRIPTION)
		values ('ACCESS_KEYWORDS', 'CSAEdit', 'CSAEDIT', 'CSA Alert Editor')
end
go

print 'Adding CSAEdit keyword to Manager, Administrator logins'
go
-- One row at a time to be as safe as possible with triggers
update Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'MANAGER'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255	-- Only if it will fit
update Users
	set AccessKeywords = AccessKeywords + ',CSAEdit'
	where UserId = 'ADMINISTRATOR'
		and upper(AccessKeywords) not like '%CSAEDIT%'
		and len(AccessKeywords) + 8 < 255	-- Only if it will fit
go

print 'Creating table of TaskCentre servers, logins, passwords (for Xtender version)'
go
if not exists (select * from Gen_Tables where TABLE_NAME = 'CSA_TASKCENTRE_SERVERS')
begin
	insert Gen_Tables (TABLE_NAME, CODE, UPPER_CODE, SUBSTITUTE, DESCRIPTION)
		values ('CSA_TASKCENTRE_SERVERS', 'ServerAlias', 'SERVERALIAS', 'RealServerName;aTCLoginForCSA;TCPassword',
			'TaskCentre server for use by CSA. Code = nickname referenced by alert. Expansion ='
			+ ' server;user;password to be used by CSA separated by semicolons.')
end
go

print 'Creating configuration items (iPart version)'
go
if not exists (select * from SystemConfig where SystemConfigKey = '42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF')
begin
	print '-- adding RSM.CSA.License'

	exec asi_InsertSystemConfig 'Organization', 'RSMCustomSettings', 'MANAGER',
		'42FDD0CF-9892-4007-9FEC-B2E42FB0AFCF', 'RSM.CSA.License',
		'85F7051F-0D92-4802-A6CC-74E9B54EA825', 7, 6, 40,
		'License key for Customer Service Alerts iParts',
		'', 100
end
else
begin
	print '-- RSM.CSA.License already exists'
end
go


print 'Creating table tcsa_EventsForTaskCentre (iPart version)'
go
if object_id('tcsa_EventsForTaskCentre') is not NULL
begin
	if (select rows from sysindexes where id = object_id('tcsa_EventsForTaskCentre') and indid < 2) > 0
	begin
		print '***Not deleting - data exists'
	end
	else
	begin
		drop table tcsa_EventsForTaskCentre
	end
end
go
if object_id('tcsa_EventsForTaskCentre') is NULL
begin
	create table tcsa_EventsForTaskCentre
	(
		-- Automatically assigned properties
		Seqn int identity,
		InsertedDate datetime not NULL default getdate(),
		InsertedBy nvarchar(128) not NULL default user_name(),
		OriginDatabase nvarchar(128) not NULL default db_name(),
		
		-- Populated by CSA
		ServerAlias varchar(60) not NULL,			-- Server alias, for compatibility with CSA desktop
		TaskId int not NULL,						-- Task which will handle this event
		UserName varchar(60) not NULL,				-- User who clicked the Action button
		SelectedId varchar(50) not NULL,			-- Contact to which the action applies
		Param1 nvarchar(255) not NULL default '',	-- Optional parameter 1
		Param2 nvarchar(255) not NULL default '',	-- Optional parameter 2
		
		-- Defaulted during insert, task may update
		ProcessedDate datetime NULL,
		ProcessedStatus int not NULL default 0,
		ProcessedMessage nvarchar(255) not NULL default ''
	)
end
go
if object_id('pkcsa_EventsForTaskCentre') is NULL
begin
	alter table tcsa_EventsForTaskCentre
		add constraint pkcsa_EventsForTaskCentre
		primary key clustered (Seqn)
end
go
if not exists (select * from sysindexes where id = object_id('tcsa_EventsForTaskCentre')
		and name = 'icsa_EventsForTaskCentreSelectedId')
begin
	create index icsa_EventsForTaskCentreSelectedId
		on tcsa_EventsForTaskCentre(SelectedId)
end
go
grant select, insert, update, delete, references, view definition on tcsa_EventsForTaskCentre to IMIS
grant select on tcsa_EventsForTaskCentre to public
go
print 'Finished.'